@@ -5,6 +5,7 @@ CFF dictionary data and Type1/Type2 CharStrings.
 from __future__ import print_function, division, absolute_import
 from fontTools.misc.py23 import *
 from fontTools.misc.fixedTools import fixedToFloat
+from fontTools.pens.boundsPen import BoundsPen
 import struct
 import logging
 
@@ -93,10 +94,6 @@ realNibblesDict = {v:i for i,v in enumerate(realNibbles)}
 maxOpStack = 193
 
 
-class ByteCodeBase(object):
-	pass
-
-
 def buildOperatorDict(operatorList):
 	oper = {}
 	opc = {}
@@ -124,7 +121,7 @@ t2Operators = [
 	(10,		'callsubr'),
 	(11,		'return'),
 	(14,		'endchar'),
-
+	(15,		'vsindex'),
 	(16,		'blend'),
 	(18,		'hstemhm'),
 	(19,		'hintmask'),
@@ -170,22 +167,6 @@ t2Operators = [
 	((12, 37),	'flex1'),
 ]
 
-CFF2Operators = [entry for entry in t2Operators if ((isinstance(entry[0], int) or
-											(entry[0][1] < 34)) and (not entry[0] in [11,14])) ]
-CFF2Operators.extend(
-[
-	(15,		'vsindex'),
-	(16,		'blend'),
-])
-
-def sortOps(entry):
-	opCode = entry[0]
-	if isinstance(opCode, int):
-		opCode = (opCode,)
-	return opCode
-
-CFF2Operators.sort(key=sortOps)
-
 def getIntEncoder(format):
 	if format == "cff":
 		fourByteOp = bytechr(29)
@@ -267,11 +248,12 @@ class CharStringCompileError(Exception): pass
 
 class SimpleT2Decompiler(object):
 
-	def __init__(self, localSubrs, globalSubrs):
+	def __init__(self, localSubrs, globalSubrs, private=None):
 		self.localSubrs = localSubrs
 		self.localBias = calcSubrBias(localSubrs)
 		self.globalSubrs = globalSubrs
 		self.globalBias = calcSubrBias(globalSubrs)
+		self.private = private
 		self.reset()
 
 	def reset(self):
@@ -279,11 +261,23 @@ class SimpleT2Decompiler(object):
 		self.operandStack = []
 		self.hintCount = 0
 		self.hintMaskBytes = 0
+		self.numRegions = 0
 
 	def check_program(self, program):
-		assert program, "illegal CharString: decompiled to empty program"
-		assert program[-1] in ("endchar", "return", "callsubr", "callgsubr",
-				"seac"), "illegal CharString"
+		if not hasattr(self, 'private') or self.private is None:
+			# Type 1 charstrings don't have self.private.
+			# Type2 CFF charstrings may have self.private == None.
+			# In both cases, they are not CFF2 charstrings
+			isCFF2 = False
+		else:
+			isCFF2 = self.private._isCFF2
+		if isCFF2:
+			if program:
+				assert program[-1] not in ("seac",), "illegal CharString Terminator"
+		else:
+			assert program, "illegal CharString: decompiled to empty program"
+			assert program[-1] in ("endchar", "return", "callsubr", "callgsubr",
+					"seac"), "illegal CharString"
 
 	def execute(self, charString):
 		self.callingStack.append(charString)
@@ -418,20 +412,8 @@ class SimpleT2Decompiler(object):
 	def op_roll(self, index):
 		raise NotImplementedError
 
-
-class SimpleCFF2Decompiler(SimpleT2Decompiler):
-	def __init__(self, localSubrs, globalSubrs, private = None):
-		super(SimpleCFF2Decompiler, self).__init__(localSubrs, globalSubrs)
-		self.private = private
-
-	def reset(self):
-		super(SimpleCFF2Decompiler, self).reset()
-		self.numRegions = 0
-
-	def check_program(self, program):
-		if program:
-			assert program[-1] not in ("seac"), "illegal CharString Terminator"
-
+	# TODO(behdad): move to T2OutlineExtractor and add a 'setVariation'
+	# method that takes VarStoreData and a location
 	def op_blend(self, index):
 		if self.numRegions == 0:
 			self.numRegions = self.private.getNumRegions()
@@ -440,13 +422,11 @@ class SimpleCFF2Decompiler(SimpleT2Decompiler):
 		blendArgs = self.operandStack[-numOps:]
 		del self.operandStack[:-(numOps-numBlends)] # Leave the default operands on the stack.
 
-
 	def op_vsindex(self, index):
 		vi = self.pop()
 		self.numRegions = self.private.getNumRegions(vi)
 
 
-
 t1Operators = [
 #	opcode		name
 	(1,		'hstem'),
@@ -815,9 +795,6 @@ class T2OutlineExtractor(T2WidthExtractor):
 		self.rCurveTo((dxa, 0), (dxb, dyb), (dxc, dyc))
 		return args
 
-class CFF2OutlineExtractor(T2OutlineExtractor):
-	pass
-
 class T1OutlineExtractor(T2OutlineExtractor):
 
 	def __init__(self, pen, subrs):
@@ -960,7 +937,7 @@ class T1OutlineExtractor(T2OutlineExtractor):
 	def op_vstem3(self, index):
 		self.popall()  # XXX
 
-class T2CharString(ByteCodeBase):
+class T2CharString(object):
 
 	operandEncoding = t2OperandEncoding
 	operators, opcodes = buildOperatorDict(t2Operators)
@@ -991,7 +968,7 @@ class T2CharString(ByteCodeBase):
 		if not self.needsDecompilation():
 			return
 		subrs = getattr(self.private, "Subrs", [])
-		decompiler = self.decompilerClass(subrs, self.globalSubrs)
+		decompiler = self.decompilerClass(subrs, self.globalSubrs, self.private)
 		decompiler.execute(self)
 
 	def draw(self, pen):
@@ -1001,17 +978,25 @@ class T2CharString(ByteCodeBase):
 		extractor.execute(self)
 		self.width = extractor.width
 
-	def check_program(self, program):
-		assert self.program, "illegal CharString: decompiled to empty program"
-		assert self.program[-1] in ("endchar", "return", "callsubr", "callgsubr",
-				"seac"), "illegal CharString"
+	def calcBounds(self):
+		boundsPen = BoundsPen(None)
+		self.draw(boundsPen)
+		return boundsPen.bounds
 
-	def compile(self):
+	def check_program(self, program, isCFF2=False):
+		if isCFF2:
+			if self.program:
+				assert self.program[-1] not in ("seac",), "illegal CFF2 CharString Termination"
+		else:
+			assert self.program, "illegal CharString: decompiled to empty program"
+			assert self.program[-1] in ("endchar", "return", "callsubr", "callgsubr", "seac"), "illegal CharString"
+
+	def compile(self, isCFF2=False):
 		if self.bytecode is not None:
 			return
 		opcodes = self.opcodes
 		program = self.program
-		self.check_program(program)
+		self.check_program(program, isCFF2=isCFF2)
 		bytecode = []
 		encodeInt = self.getIntEncoder()
 		encodeFixed = self.getFixedEncoder()
@@ -1042,6 +1027,11 @@ class T2CharString(ByteCodeBase):
 			raise
 		self.setBytecode(bytecode)
 
+		if isCFF2:
+			# If present, remove return and endchar operators.
+			if self.bytecode and (byteord(self.bytecode[-1]) in (11, 14)):
+				self.bytecode = self.bytecode[:-1]
+
 	def needsDecompilation(self):
 		return self.bytecode is not None
 
@@ -1146,36 +1136,6 @@ class T2CharString(ByteCodeBase):
 				program.append(token)
 		self.setProgram(program)
 
-class CFF2CharString(T2CharString):
-
-	operandEncoding = t2OperandEncoding
-	operators, opcodes = buildOperatorDict(CFF2Operators)
-	decompilerClass = SimpleCFF2Decompiler
-	outlineExtractor = CFF2OutlineExtractor
-
-	def check_program(self, program):
-		#assert self.program, "illegal CharString: decompiled to empty program"
-		# Empty program is OK for CFF2 Charstring - same a T2 with only an endchar.
-		if self.program:
-			assert self.program[-1] not in ("seac"), "illegal CFF2 CharString Termination"
-
-	def compile(self):
-		# Remove endchar, if there is one.
-		super(CFF2CharString, self).compile()
-		if self.bytecode and (byteord(self.bytecode[-1]) in (11, 14)):
-			self.bytecode = self.bytecode[:-1]
-
-	def decompile(self):
-		# Remove endchar, if there is one.
-		if not self.needsDecompilation():
-			return
-		subrs = getattr(self.private, "Subrs", [])
-		decompiler = self.decompilerClass(subrs, self.globalSubrs, self.private)
-		decompiler.execute(self)
-
-class CFF2SubrCharString(CFF2CharString):
-	pass
-
 
 class T1CharString(T2CharString):
 
@@ -1214,14 +1174,15 @@ class T1CharString(T2CharString):
 		self.width = extractor.width
 
 
-class DictDecompiler(ByteCodeBase):
+class DictDecompiler(object):
 
 	operandEncoding = cffDictOperandEncoding
 
-	def __init__(self, strings):
+	def __init__(self, strings, parent=None):
 		self.stack = []
 		self.strings = strings
 		self.dict = {}
+		self.parent = parent
 
 	def getDict(self):
 		assert len(self.stack) == 0, "non-empty stack"
