@@ -90,9 +90,11 @@ class TTFont(object):
 
 		If the recalcBBoxes argument is false, a number of things will *not*
 		be recalculated upon save/compile:
-			1) glyph bounding boxes
-			2) maxp font bounding box
-			3) hhea min/max values
+			1) 'glyf' glyph bounding boxes
+			2) 'CFF ' font bounding box
+			3) 'head' font bounding box
+			4) 'hhea' min/max values
+			5) 'vhea' min/max values
 		(1) is needed for certain kinds of CJK fonts (ask Werner Lemberg ;-).
 		Additionally, upon importing an TTX file, this option cause glyphs
 		to be compiled right away. This should reduce memory consumption
@@ -203,6 +205,9 @@ class TTFont(object):
 			# assume "file" is a writable file object
 			closeStream = False
 
+		if self.recalcTimestamp and 'head' in self:
+			self['head']  # make sure 'head' is loaded so the recalculation is actually done
+
 		tags = list(self.keys())
 		if "GlyphOrder" in tags:
 			tags.remove("GlyphOrder")
@@ -314,8 +319,9 @@ class TTFont(object):
 			progress.set((i + 1))
 		writer.endtag("ttFont")
 		writer.newline()
-		# close if 'fileOrPath' is a path; leave it open if it's a file
-		if not hasattr(fileOrPath, "write"):
+		# close if 'fileOrPath' is a path; leave it open if it's a file.
+		# The special string "-" means standard output so leave that open too
+		if not hasattr(fileOrPath, "write") and fileOrPath != "-":
 			writer.close()
 
 	def _tableToXML(self, writer, tag, progress, quiet=None):
@@ -555,7 +561,7 @@ class TTFont(object):
 
 	def getGlyphNames(self):
 		"""Get a list of glyph names, sorted alphabetically."""
-		glyphNames = sorted(self.getGlyphOrder()[:])
+		glyphNames = sorted(self.getGlyphOrder())
 		return glyphNames
 
 	def getGlyphNames2(self):
@@ -675,14 +681,18 @@ class TTFont(object):
 		have a .draw() method that supports the Pen protocol, and will
 		have an attribute named 'width'.
 
-		If the font is CFF-based, the outlines will be taken from the 'CFF '
-		table. Otherwise the outlines will be taken from the 'glyf' table.
-		If the font contains both a 'CFF ' and a 'glyf' table, you can use
-		the 'preferCFF' argument to specify which one should be taken.
+		If the font is CFF-based, the outlines will be taken from the 'CFF ' or
+		'CFF2' tables. Otherwise the outlines will be taken from the 'glyf' table.
+		If the font contains both a 'CFF '/'CFF2' and a 'glyf' table, you can use
+		the 'preferCFF' argument to specify which one should be taken. If the
+		font contains both a 'CFF ' and a 'CFF2' table, the latter is taken.
 		"""
 		glyphs = None
-		if (preferCFF and "CFF " in self) or "glyf" not in self:
-			glyphs = _TTGlyphSet(self, list(self["CFF "].cff.values())[0].CharStrings, _TTGlyphCFF)
+		if (preferCFF and any(tb in self for tb in ["CFF ", "CFF2"]) or
+		   ("glyf" not in self and any(tb in self for tb in ["CFF ", "CFF2"]))):
+			table_tag = "CFF2" if "CFF2" in self else "CFF "
+			glyphs = _TTGlyphSet(self,
+			    list(self[table_tag].cff.values())[0].CharStrings, _TTGlyphCFF)
 
 		if glyphs is None and "glyf" in self:
 			glyphs = _TTGlyphSet(self, self["glyf"], _TTGlyphGlyf)
@@ -702,6 +712,7 @@ class _TTGlyphSet(object):
 	def __init__(self, ttFont, glyphs, glyphType):
 		self._glyphs = glyphs
 		self._hmtx = ttFont['hmtx']
+		self._vmtx = ttFont['vmtx'] if 'vmtx' in ttFont else None
 		self._glyphType = glyphType
 
 	def keys(self):
@@ -713,7 +724,10 @@ class _TTGlyphSet(object):
 	__contains__ = has_key
 
 	def __getitem__(self, glyphName):
-		return self._glyphType(self, self._glyphs[glyphName], self._hmtx[glyphName])
+		horizontalMetrics = self._hmtx[glyphName]
+		verticalMetrics = self._vmtx[glyphName] if self._vmtx else None
+		return self._glyphType(
+			self, self._glyphs[glyphName], horizontalMetrics, verticalMetrics)
 
 	def get(self, glyphName, default=None):
 		try:
@@ -725,13 +739,21 @@ class _TTGlyph(object):
 
 	"""Wrapper for a TrueType glyph that supports the Pen protocol, meaning
 	that it has a .draw() method that takes a pen object as its only
-	argument. Additionally there is a 'width' attribute.
+	argument. Additionally there are 'width' and 'lsb' attributes, read from
+	the 'hmtx' table.
+
+	If the font contains a 'vmtx' table, there will also be 'height' and 'tsb'
+	attributes.
 	"""
 
-	def __init__(self, glyphset, glyph, metrics):
+	def __init__(self, glyphset, glyph, horizontalMetrics, verticalMetrics=None):
 		self._glyphset = glyphset
 		self._glyph = glyph
-		self.width, self.lsb = metrics
+		self.width, self.lsb = horizontalMetrics
+		if verticalMetrics:
+			self.height, self.tsb = verticalMetrics
+		else:
+			self.height, self.tsb = None, None
 
 	def draw(self, pen):
 		"""Draw the glyph onto Pen. See fontTools.pens.basePen for details
