@@ -31,8 +31,8 @@ log = logging.getLogger(__name__)
 
 class TupleVariation(object):
 	def __init__(self, axes, coordinates):
-		self.axes = axes
-		self.coordinates = coordinates
+		self.axes = axes.copy()
+		self.coordinates = coordinates[:]
 
 	def __repr__(self):
 		axes = ",".join(sorted(["%s=%s" % (name, value) for (name, value) in self.axes.items()]))
@@ -65,7 +65,7 @@ class TupleVariation(object):
 		for axis in axisTags:
 			value = self.axes.get(axis)
 			if value is not None:
-				minValue, value, maxValue = value
+				minValue, value, maxValue = (float(v) for v in value)
 				defaultMinValue = min(value, 0.0)  # -0.3 --> -0.3; 0.7 --> 0.0
 				defaultMaxValue = max(value, 0.0)  # -0.3 -->  0.0; 0.7 --> 0.7
 				if minValue == defaultMinValue and maxValue == defaultMaxValue:
@@ -366,7 +366,7 @@ class TupleVariation(object):
 		assert runLength >= 1 and runLength <= 64
 		stream.write(bytechr(runLength - 1))
 		for i in range(offset, pos):
-			stream.write(struct.pack('b', deltas[i]))
+			stream.write(struct.pack('b', round(deltas[i])))
 		return pos
 
 	@staticmethod
@@ -400,7 +400,7 @@ class TupleVariation(object):
 		assert runLength >= 1 and runLength <= 64
 		stream.write(bytechr(DELTAS_ARE_WORDS | (runLength - 1)))
 		for i in range(offset, pos):
-			stream.write(struct.pack('>h', deltas[i]))
+			stream.write(struct.pack('>h', round(deltas[i])))
 		return pos
 
 	@staticmethod
