@@ -728,7 +728,7 @@ _MAC_LANGUAGES = {
     37: 'ro',
     38: 'cz',
     39: 'sk',
-    40: 'si',
+    40: 'sl',
     41: 'yi',
     42: 'sr',
     43: 'mk',
