@@ -36,26 +36,10 @@ class BaseTTXConverter(DefaultTable):
 
 	def decompile(self, data, font):
 		from . import otTables
-		cachingStats = None if True else {}
-		class GlobalState(object):
-			def __init__(self, tableType, cachingStats):
-				self.tableType = tableType
-				self.cachingStats = cachingStats
-		globalState = GlobalState(tableType=self.tableTag,
-					cachingStats=cachingStats)
-		reader = OTTableReader(data, globalState)
+		reader = OTTableReader(data, tableTag=self.tableTag)
 		tableClass = getattr(otTables, self.tableTag)
 		self.table = tableClass()
 		self.table.decompile(reader, font)
-		if cachingStats:
-			stats = sorted([(v, k) for k, v in cachingStats.items()])
-			stats.reverse()
-			log.debug("cachingStats for %s", self.tableTag)
-			for v, k in stats:
-				if v < 2:
-					break
-				log.debug("%s %s", v, k)
-			log.debug("--- %s", len(stats))
 
 	def compile(self, font):
 		""" Create a top-level OTFWriter for the GPOS/GSUB table.
@@ -78,15 +62,11 @@ class BaseTTXConverter(DefaultTable):
 
 				If a lookup subtable overflows an offset, we have to start all over.
 		"""
-		class GlobalState(object):
-			def __init__(self, tableType):
-				self.tableType = tableType
-		globalState = GlobalState(tableType=self.tableTag)
 		overflowRecord = None
 
 		while True:
 			try:
-				writer = OTTableWriter(globalState)
+				writer = OTTableWriter(tableTag=self.tableTag)
 				self.table.compile(writer, font)
 				return writer.getAllData()
 
@@ -124,14 +104,14 @@ class OTTableReader(object):
 
 	"""Helper class to retrieve data from an OpenType table."""
 
-	__slots__ = ('data', 'offset', 'pos', 'globalState', 'localState')
+	__slots__ = ('data', 'offset', 'pos', 'localState', 'tableTag')
 
-	def __init__(self, data, globalState={}, localState=None, offset=0):
+	def __init__(self, data, localState=None, offset=0, tableTag=None):
 		self.data = data
 		self.offset = offset
 		self.pos = offset
-		self.globalState = globalState
 		self.localState = localState
+		self.tableTag = tableTag
 
 	def advance(self, count):
 		self.pos += count
@@ -140,13 +120,13 @@ class OTTableReader(object):
 		self.pos = pos
 
 	def copy(self):
-		other = self.__class__(self.data, self.globalState, self.localState, self.offset)
+		other = self.__class__(self.data, self.localState, self.offset, self.tableTag)
 		other.pos = self.pos
 		return other
 
 	def getSubReader(self, offset):
 		offset = self.offset + offset
-		return self.__class__(self.data, self.globalState, self.localState, offset)
+		return self.__class__(self.data, self.localState, offset, self.tableTag)
 
 	def readUShort(self):
 		pos = self.pos
@@ -237,11 +217,11 @@ class OTTableWriter(object):
 
 	"""Helper class to gather and assemble data for OpenType tables."""
 
-	def __init__(self, globalState, localState=None):
+	def __init__(self, localState=None, tableTag=None):
 		self.items = []
 		self.pos = None
-		self.globalState = globalState
 		self.localState = localState
+		self.tableTag = tableTag
 		self.longOffset = False
 		self.parent = None
 
@@ -253,17 +233,19 @@ class OTTableWriter(object):
 	def __getitem__(self, name):
 		return self.localState[name]
 
+	def __delitem__(self, name):
+		del self.localState[name]
+
 	# assembler interface
 
 	def getDataLength(self):
 		"""Return the length of this table in bytes, without subtables."""
 		l = 0
 		for item in self.items:
-			if hasattr(item, "getData") or hasattr(item, "getCountData"):
-				if item.longOffset:
-					l = l + 4  # sizeof(ULong)
-				else:
-					l = l + 2  # sizeof(UShort)
+			if hasattr(item, "getCountData"):
+				l += item.size
+			elif hasattr(item, "getData"):
+				l += 4 if item.longOffset else 2
 			else:
 				l = l + len(item)
 		return l
@@ -424,7 +406,7 @@ class OTTableWriter(object):
 	# interface for gathering data, as used by table.compile()
 
 	def getSubWriter(self):
-		subwriter = self.__class__(self.globalState, self.localState)
+		subwriter = self.__class__(self.localState, self.tableTag)
 		subwriter.parent = self # because some subtables have idential values, we discard
 					# the duplicates under the getAllData method. Hence some
 					# subtable writers can have more than one parent writer.
@@ -466,8 +448,8 @@ class OTTableWriter(object):
 	def writeSubTable(self, subWriter):
 		self.items.append(subWriter)
 
-	def writeCountReference(self, table, name, size=2):
-		ref = CountReference(table, name, size=size)
+	def writeCountReference(self, table, name, size=2, value=None):
+		ref = CountReference(table, name, size=size, value=value)
 		self.items.append(ref)
 		return ref
 
@@ -509,15 +491,17 @@ class OTTableWriter(object):
 						LookupListIndex = p1.parent.repeatIndex
 						SubTableIndex = p1.repeatIndex
 
-		return OverflowErrorRecord( (self.globalState.tableType, LookupListIndex, SubTableIndex, itemName, itemIndex) )
+		return OverflowErrorRecord( (self.tableTag, LookupListIndex, SubTableIndex, itemName, itemIndex) )
 
 
 class CountReference(object):
 	"""A reference to a Count value, not a count of references."""
-	def __init__(self, table, name, size=None):
+	def __init__(self, table, name, size=None, value=None):
 		self.table = table
 		self.name = name
 		self.size = size
+		if value is not None:
+			self.setValue(value)
 	def setValue(self, value):
 		table = self.table
 		name = self.name
@@ -526,16 +510,17 @@ class CountReference(object):
 		else:
 			assert table[name] == value, (name, table[name], value)
 	def getCountData(self):
-		assert self.size in (2, 4)
 		v = self.table[self.name]
 		if v is None: v = 0
-		return packUShort(v) if self.size == 2 else packULong(v)
+		return {1:packUInt8, 2:packUShort, 4:packULong}[self.size](v)
 
 
+def packUInt8 (value):
+	return struct.pack(">B", value)
+
 def packUShort(value):
 	return struct.pack(">H", value)
 
-
 def packULong(value):
 	assert 0 <= value < 0x100000000, value
 	return struct.pack(">L", value)
@@ -585,20 +570,46 @@ class BaseTable(object):
 	def getConverterByName(self, name):
 		return self.convertersByName[name]
 
+	def populateDefaults(self, propagator=None):
+		for conv in self.getConverters():
+			if conv.repeat:
+				if not hasattr(self, conv.name):
+					setattr(self, conv.name, [])
+				countValue = len(getattr(self, conv.name)) - conv.aux
+				try:
+					count_conv = self.getConverterByName(conv.repeat)
+					setattr(self, conv.repeat, countValue)
+				except KeyError:
+					# conv.repeat is a propagated count
+					if propagator and conv.repeat in propagator:
+						propagator[conv.repeat].setValue(countValue)
+			else:
+				if conv.aux and not eval(conv.aux, None, self.__dict__):
+					continue
+				if hasattr(self, conv.name):
+					continue # Warn if it should NOT be present?!
+				if hasattr(conv, 'writeNullOffset'):
+					setattr(self, conv.name, None) # Warn?
+				#elif not conv.isCount:
+				#	# Warn?
+				#	pass
+
 	def decompile(self, reader, font):
 		self.readFormat(reader)
 		table = {}
 		self.__rawTable = table  # for debugging
-		converters = self.getConverters()
-		for conv in converters:
+		for conv in self.getConverters():
 			if conv.name == "SubTable":
-				conv = conv.getConverter(reader.globalState.tableType,
+				conv = conv.getConverter(reader.tableTag,
 						table["LookupType"])
 			if conv.name == "ExtSubTable":
-				conv = conv.getConverter(reader.globalState.tableType,
+				conv = conv.getConverter(reader.tableTag,
 						table["ExtensionLookupType"])
 			if conv.name == "FeatureParams":
 				conv = conv.getConverter(reader["FeatureTag"])
+			if conv.name == "SubStruct":
+				conv = conv.getConverter(reader.tableTag,
+				                         table["MorphType"])
 			if conv.repeat:
 				if conv.repeat in table:
 					countValue = table[conv.repeat]
@@ -614,13 +625,20 @@ class BaseTable(object):
 				if conv.isPropagated:
 					reader[conv.name] = table[conv.name]
 
-		self.postRead(table, font)
+		if hasattr(self, 'postRead'):
+			self.postRead(table, font)
+		else:
+			self.__dict__.update(table)
 
 		del self.__rawTable  # succeeded, get rid of debugging info
 
 	def compile(self, writer, font):
 		self.ensureDecompiled()
-		table = self.preWrite(font)
+		if hasattr(self, 'preWrite'):
+			table = self.preWrite(font)
+		else:
+			table = self.__dict__.copy()
+
 
 		if hasattr(self, 'sortCoverageLast'):
 			writer.sortCoverageLast = 1
@@ -639,7 +657,7 @@ class BaseTable(object):
 					value = []
 				countValue = len(value) - conv.aux
 				if conv.repeat in table:
-					CountReference(table, conv.repeat).setValue(countValue)
+					CountReference(table, conv.repeat, value=countValue)
 				else:
 					# conv.repeat is a propagated count
 					writer[conv.repeat].setValue(countValue)
@@ -664,8 +682,12 @@ class BaseTable(object):
 				if conv.isPropagated:
 					writer[conv.name] = ref
 			elif conv.isLookupType:
-				ref = writer.writeCountReference(table, conv.name, conv.staticSize)
-				table[conv.name] = None
+				# We make sure that subtables have the same lookup type,
+				# and that the type is the same as the one set on the
+				# Lookup object, if any is set.
+				if conv.name not in table:
+					table[conv.name] = None
+				ref = writer.writeCountReference(table, conv.name, conv.staticSize, table[conv.name])
 				writer['LookupType'] = ref
 			else:
 				if conv.aux and not eval(conv.aux, None, table):
@@ -685,12 +707,6 @@ class BaseTable(object):
 	def writeFormat(self, writer):
 		pass
 
-	def postRead(self, table, font):
-		self.__dict__.update(table)
-
-	def preWrite(self, font):
-		return self.__dict__.copy()
-
 	def toXML(self, xmlWriter, font, attrs=None, name=None):
 		tableName = name if name else self.__class__.__name__
 		if attrs is None:
@@ -766,7 +782,6 @@ class FormatSwitchingBaseTable(BaseTable):
 
 	def readFormat(self, reader):
 		self.Format = reader.readUShort()
-		assert self.Format != 0, (self, reader.pos, len(reader.data))
 
 	def writeFormat(self, writer):
 		writer.writeUShort(self.Format)
@@ -868,11 +883,17 @@ class ValueRecord(object):
 
 	# see ValueRecordFactory
 
-	def __init__(self, valueFormat=None):
+	def __init__(self, valueFormat=None, src=None):
 		if valueFormat is not None:
 			for mask, name, isDevice, signed in valueRecordFormat:
 				if valueFormat & mask:
 					setattr(self, name, None if isDevice else 0)
+			if src is not None:
+				for key,val in src.__dict__.items():
+					assert hasattr(self, key)
+					setattr(self, key, val)
+		elif src is not None:
+			self.__dict__ = src.__dict__.copy()
 
 	def getFormat(self):
 		format = 0
