@@ -4,9 +4,9 @@ usage: ttx [options] inputfile1 [... inputfileN]
     TTX -- From OpenType To XML And Back
 
     If an input file is a TrueType or OpenType font file, it will be
-       dumped to an TTX file (an XML-based text format).
-    If an input file is a TTX file, it will be compiled to a TrueType
-       or OpenType font file.
+       decompiled to a TTX file (an XML-based text format).
+    If an input file is a TTX file, it will be compiled to whatever 
+       format the data is in, a TrueType or OpenType/CFF font file.
 
     Output files are created so they are unique: an existing file is
        never overwritten.
@@ -155,6 +155,8 @@ class Options(object):
 				value = value.ljust(4)
 				self.onlyTables.append(value)
 			elif option == "-x":
+				# pad with space if table tag length is less than 4
+				value = value.ljust(4)
 				self.skipTables.append(value)
 			elif option == "-s":
 				self.splitTables = True
